@@ -160,6 +160,10 @@ func (d *Decoder) decode(dst interface{}) error {
 	if v.Kind() != reflect.Ptr {
 		return fmt.Errorf("msgpack: Decode(nonsettable %T)", dst)
 	}
+	v = v.Elem()
+	if !v.IsValid() {
+		return fmt.Errorf("msgpack: Decode(nonsettable %T)", dst)
+	}
 	return d.DecodeValue(v)
 }
 
@@ -217,10 +221,10 @@ func (d *Decoder) interfaceValue(v reflect.Value) error {
 	return nil
 }
 
-// Decodes value into interface. Possible value types are:
+// DecodeInterface decodes value into interface. Possible value types are:
 //   - nil,
-//   - int64,
-//   - uint64,
+//   - int64 for negative numbers,
+//   - uint64 for positive numbers,
 //   - bool,
 //   - float32 and float64,
 //   - string,
@@ -233,10 +237,10 @@ func (d *Decoder) DecodeInterface() (interface{}, error) {
 	}
 
 	if codes.IsFixedNum(c) {
-		if c >= 0 {
-			return d.uint(c)
+		if int8(c) < 0 {
+			return d.int(c)
 		}
-		return d.int(c)
+		return d.uint(c)
 	}
 	if codes.IsFixedMap(c) {
 		d.r.UnreadByte()
@@ -325,7 +329,7 @@ func (d *Decoder) Skip() error {
 
 // peekCode returns the next Msgpack code. See
 // https://github.com/msgpack/msgpack/blob/master/spec.md#formats for details.
-func (d *Decoder) peekCode() (code byte, err error) {
+func (d *Decoder) PeekCode() (code byte, err error) {
 	code, err = d.r.ReadByte()
 	if err != nil {
 		return 0, err
@@ -334,7 +338,7 @@ func (d *Decoder) peekCode() (code byte, err error) {
 }
 
 func (d *Decoder) gotNilCode() bool {
-	code, err := d.peekCode()
+	code, err := d.PeekCode()
 	return err == nil && code == codes.Nil
 }
 
