@@ -1,6 +1,8 @@
 package msgpack_test
 
 import (
+	"fmt"
+	"net/url"
 	"reflect"
 	"testing"
 	"time"
@@ -47,27 +49,27 @@ func (setptr *intSet) DecodeMsgpack(dec *msgpack.Decoder) error {
 
 //------------------------------------------------------------------------------
 
-type compactEncoding struct {
+type CompactEncodingTest struct {
 	str string
-	ref *compactEncoding
+	ref *CompactEncodingTest
 	num int
 }
 
 var (
-	_ msgpack.CustomEncoder = &compactEncoding{}
-	_ msgpack.CustomDecoder = &compactEncoding{}
+	_ msgpack.CustomEncoder = (*CompactEncodingTest)(nil)
+	_ msgpack.CustomDecoder = (*CompactEncodingTest)(nil)
 )
 
-func (s *compactEncoding) EncodeMsgpack(enc *msgpack.Encoder) error {
+func (s *CompactEncodingTest) EncodeMsgpack(enc *msgpack.Encoder) error {
 	return enc.Encode(s.str, s.ref, s.num)
 }
 
-func (s *compactEncoding) DecodeMsgpack(dec *msgpack.Decoder) error {
+func (s *CompactEncodingTest) DecodeMsgpack(dec *msgpack.Decoder) error {
 	return dec.Decode(&s.str, &s.ref, &s.num)
 }
 
-type compactEncodingFieldValue struct {
-	Field compactEncoding
+type CompactEncodingFieldTest struct {
+	Field CompactEncodingTest
 }
 
 //------------------------------------------------------------------------------
@@ -90,15 +92,23 @@ type binTest struct {
 
 var binTests = []binTest{
 	{nil, []byte{codes.Nil}},
+
+	{[]byte(nil), []byte{codes.Nil}},
+	{[]byte{1, 2, 3}, []byte{codes.Bin8, 0x3, 0x1, 0x2, 0x3}},
+	{[3]byte{1, 2, 3}, []byte{codes.Bin8, 0x3, 0x1, 0x2, 0x3}},
+
 	{OmitEmptyTest{}, []byte{codes.FixedMapLow}},
 
 	{intSet{}, []byte{codes.FixedArrayLow}},
 	{intSet{8: struct{}{}}, []byte{codes.FixedArrayLow | 1, 0x8}},
 
-	{&compactEncoding{}, []byte{codes.FixedStrLow, codes.Nil, 0x0}},
+	{map[string]string(nil), []byte{codes.Nil}},
+	{map[string]string(nil), []byte{codes.Nil}},
+
+	{&CompactEncodingTest{}, []byte{codes.FixedStrLow, codes.Nil, 0x0}},
 	{
-		&compactEncoding{"n", &compactEncoding{"o", nil, 7}, 6},
-		[]byte{codes.FixedStrLow | 1, 'n', codes.FixedStrLow | 1, 'o', codes.Nil, 0x7, 0x6},
+		&CompactEncodingTest{"a", &CompactEncodingTest{"b", nil, 7}, 6},
+		[]byte{codes.FixedStrLow | 1, 'a', codes.FixedStrLow | 1, 'b', codes.Nil, 0x7, 0x6},
 	},
 }
 
@@ -128,14 +138,29 @@ func TestBin(t *testing.T) {
 		}
 
 		if !reflect.DeepEqual(b, test.wanted) {
-			t.Fatalf("% x != % x", b, test.wanted)
+			t.Fatalf("%q != %q (in=%#v)", b, test.wanted, test.in)
 		}
 	}
 }
 
 //------------------------------------------------------------------------------
 
-type embeddedTime struct {
+type unexported struct {
+	Foo string
+}
+
+type Exported struct {
+	Bar string
+}
+
+type EmbedingTest struct {
+	unexported
+	Exported
+}
+
+//------------------------------------------------------------------------------
+
+type TimeEmbedingTest struct {
 	time.Time
 }
 
@@ -145,7 +170,7 @@ type (
 	stringSlice []string
 )
 
-type testStruct struct {
+type StructTest struct {
 	F1 stringSlice
 	F2 []string
 }
@@ -153,22 +178,29 @@ type testStruct struct {
 type typeTest struct {
 	*testing.T
 
-	in     interface{}
-	out    interface{}
-	encErr string
-	decErr string
+	in      interface{}
+	out     interface{}
+	encErr  string
+	decErr  string
+	wantnil bool
+	wanted  interface{}
+}
+
+func (t typeTest) String() string {
+	return fmt.Sprintf("in=%#v, out=%#v", t.in, t.out)
 }
 
 func (t *typeTest) assertErr(err error, s string) {
 	if err == nil {
-		t.Fatalf("got %v, wanted %q", err, s)
+		t.Fatalf("got %v error, wanted %q", err, s)
 	}
 	if err.Error() != s {
-		t.Fatalf("got %q, wanted %q", err, s)
+		t.Fatalf("got %q error, wanted %q", err, s)
 	}
 }
 
 var (
+	uri, _    = url.Parse("https://github.com/vmihailenco/msgpack")
 	typeTests = []typeTest{
 		{in: make(chan bool), encErr: "msgpack: Encode(unsupported chan bool)"},
 
@@ -177,12 +209,30 @@ var (
 		{in: nil, out: (*int)(nil), decErr: "msgpack: Decode(nonsettable *int)"},
 		{in: nil, out: new(chan bool), decErr: "msgpack: Decode(unsupported chan bool)"},
 
-		{in: []int(nil), out: new([]int)},
+		{in: nil, out: new(int), wanted: int(0)},
+		{in: nil, out: new(*int), wantnil: true},
+
+		{in: nil, out: new([]int), wantnil: true},
+		{in: nil, out: &[]int{1, 2}, wantnil: true},
 		{in: make([]int, 0), out: new([]int)},
 		{in: make([]int, 1000), out: new([]int)},
+
 		{in: []interface{}{uint64(1), "hello"}, out: new([]interface{})},
 		{in: map[string]interface{}{"foo": nil}, out: new(map[string]interface{})},
 
+		{in: nil, out: new([]byte), wantnil: true},
+		{in: []byte(nil), out: new([]byte)},
+		{in: []byte{1, 2, 3}, out: new([]byte)},
+
+		{in: nil, out: new([3]byte), wanted: [3]byte{}},
+		{in: [3]byte{1, 2, 3}, out: new([3]byte)},
+		{in: [3]byte{1, 2, 3}, out: new([2]byte), wanted: [2]byte{1, 2}},
+
+		{in: nil, out: new(map[string]string), wantnil: true},
+		{in: nil, out: new(map[int]int), wantnil: true},
+		{in: nil, out: &map[string]string{"foo": "bar"}, wantnil: true},
+		{in: nil, out: &map[int]int{1: 2}, wantnil: true},
+
 		{in: stringSlice{"foo", "bar"}, out: new(stringSlice)},
 		{in: []stringAlias{"hello"}, out: new([]stringAlias)},
 		{in: []uint8Alias{1}, out: new([]uint8Alias)},
@@ -190,16 +240,27 @@ var (
 		{in: intSet{}, out: new(intSet)},
 		{in: intSet{8: struct{}{}}, out: new(intSet)},
 
-		{in: testStruct{stringSlice{"foo", "bar"}, []string{"hello"}}, out: new(testStruct)},
-		{in: &coderStruct{name: "hello"}, out: new(*coderStruct)},
-		{in: &embeddedTime{Time: time.Now()}, out: new(*embeddedTime)},
-
-		{in: &compactEncoding{}, out: new(compactEncoding)},
-		{in: &compactEncoding{"a", &compactEncoding{"b", nil, 1}, 2}, out: new(compactEncoding)},
+		{in: StructTest{stringSlice{"foo", "bar"}, []string{"hello"}}, out: new(StructTest)},
+		{in: TimeEmbedingTest{Time: time.Now()}, out: new(TimeEmbedingTest)},
 		{
-			in:  &compactEncodingFieldValue{Field: compactEncoding{"a", nil, 1}},
-			out: new(compactEncodingFieldValue),
+			in: EmbedingTest{
+				unexported: unexported{Foo: "hello"},
+				Exported:   Exported{Bar: "world"},
+			},
+			out: new(EmbedingTest),
 		},
+
+		{in: &CompactEncodingTest{}, out: new(CompactEncodingTest)},
+		{
+			in:  &CompactEncodingTest{"a", &CompactEncodingTest{"b", nil, 1}, 2},
+			out: new(CompactEncodingTest),
+		},
+		{
+			in:  &CompactEncodingFieldTest{Field: CompactEncodingTest{"a", nil, 1}},
+			out: new(CompactEncodingFieldTest),
+		},
+
+		{in: uri, out: new(url.URL)},
 	}
 )
 
@@ -224,7 +285,7 @@ func TestTypes(t *testing.T) {
 			continue
 		}
 		if err != nil {
-			t.Fatal(err)
+			t.Fatalf("Marshal failed: %s (in=%#v)", err, test.in)
 		}
 
 		err = msgpack.Unmarshal(b, test.out)
@@ -233,13 +294,25 @@ func TestTypes(t *testing.T) {
 			continue
 		}
 		if err != nil {
-			t.Fatal(err)
+			t.Fatalf("Unmarshal failed: %s (%s)", err, test)
+		}
+
+		if test.wantnil {
+			v := reflect.ValueOf(test.out).Elem()
+			if v.IsNil() {
+				continue
+			}
+			t.Fatalf("got %#v, wanted nil", v.Interface())
+			continue
 		}
 
-		in := deref(test.in)
 		out := deref(test.out)
-		if !reflect.DeepEqual(out, in) {
-			t.Fatalf("%#v != %#v", out, in)
+		wanted := test.wanted
+		if wanted == nil {
+			wanted = deref(test.in)
+		}
+		if !reflect.DeepEqual(out, wanted) {
+			t.Fatalf("%#v != %#v (%s)", out, wanted, test)
 		}
 	}
 }
